@@ -9,8 +9,8 @@ Created on Fri May 31 22:32:52 2019
 import pandas as pd
 
 #local imports
-from ftr_classifier.word_lists import FEATURES, WORD_LISTS, MAIN_FEATURES
-from ftr_classifier.models import MODELS
+from .word_lists import FEATURES,WORD_LISTS,ALL_FEATURES
+from .models import MODELS
 
 
 #fuck off pandas
@@ -20,7 +20,7 @@ pd.options.mode.chained_assignment = None  # default='warn'
 _MISSING = '-999'
 
 #functions
-def _append_spacy_docs(df,lang_col='language',text_col='response'):
+def _append_spacy_docs(df,lang_col,text_col):
     #create new dataframe
     dx = pd.DataFrame()
     #group by language
@@ -34,12 +34,7 @@ def _append_spacy_docs(df,lang_col='language',text_col='response'):
         dx = dx.append(dy)
     return dx
 
-def _append_last_sentence(df):
-    df = df.copy()
-    df['final_sentence'] = df.spacy_doc.apply(lambda doc: [sent for sent in doc.sents][-1])
-    return df
-
-def _clean_non_applicable(df,lang_col='language'):
+def _clean_non_applicable(df,lang_col):
     #create new dataframe
     dx = pd.DataFrame()
     #group by language
@@ -56,20 +51,10 @@ def _clean_non_applicable(df,lang_col='language'):
         dx = dx.append(dy)
     return dx
 
-def _present_dom(df):
-    df = df.copy()
-    #other features not pres
-    other_features = [x for x in FEATURES if x != 'present']
-    #apply
-    df['present_dom'] = [0 if any(feature == 1 for feature in df.loc[x,other_features])\
-                  else 1 if df.present[x] == 1 else 0\
-                  for x in df.index]
-    return df
-
 def _future_dom(df):
     df = df.copy()
     futures = ['go_future','will_future','future']
-    exclude = ['present','particle']
+    exclude = ['present','particle','past','present_perfect','past_perfect']
     other_features = [x for x in FEATURES if x not in futures and x not in exclude]
     for future in futures:
         df[future+'_dom'] = [0 if any(feature == 1 for feature in df.loc[x,other_features])\
@@ -77,28 +62,87 @@ def _future_dom(df):
                               for x in df.index]
     return df
 
+def _aspect_dom(df,lang_col='language'):
+    dfg = df.groupby(lang_col)
+    store = pd.DataFrame()
+    for lang,dx in dfg:
+        if lang == 'english':
+            dx['present_perfect'] = [1 if dx.present_perfect[x] == 1 and dx.past[x] == 1 and dx.future[x] == 0 else 0 for x in dx.index]
+            dx['past_perfect'] = [1 if dx.past_perfect[x] == 1 and dx.past[x] == 1 and dx.future[x] == 0 else 0 for x in dx.index]
+            store = store.append(dx)
+        elif lang == 'dutch':
+            dx['present_perfect'] = [1 if dx.present_perfect[x] == 1 and dx.past_participle[x] == 1 and dx.future[x] == 0 else 0 for x in dx.index]
+            dx['past_perfect'] = [1 if dx.past_perfect[x] == 1 and dx.past_participle[x] == 1 and dx.future[x] == 0 else 0 for x in dx.index]
+            store = store.append(dx)
+        else:
+            store = store.append(dx)
+    store.index = range(store.shape[0])
+    return store
+
+def _past_dom(df):
+    df = df.copy()
+    other_features = ['verb_cert','verb_poss','future','verb_des_int','present_perfect']
+    df['past'] = [0 if any(feature == 1 for feature in df.loc[x,other_features])\
+                          else 1 if df.past[x] == 1 else 0\
+                          for x in df.index]
+    return df
+
+def _present_dom(df):
+    df = df.copy()
+    #other features not pres
+    other_features = [x for x in FEATURES if x != 'present' and x not in ['particle','past_perfect']]
+    #apply
+    df['present_dom'] = [0 if any(feature == 1 for feature in df.loc[x,other_features])\
+                          else 1 if df.present[x] == 1 \
+                          else 0\
+                          for x in df.index]
+    return df
+
+def _modal_exclude(df):
+    df = df.copy()
+    posses = [x for x in FEATURES if x.endswith('poss')]
+    certs = [x for x in FEATURES if x.endswith('cert')]
+    for cert in certs:
+        df[cert] = ['-999' if any(feature == 1 for feature in df.loc[x,posses]) and df[cert][x] == 1\
+                              else df[cert][x] for x in df.index]
+    for poss in posses:
+        df[cert] = ['-999' if any(feature == 1 for feature in df.loc[x,certs]) and df[poss][x] == 1\
+                              else df[poss][x] for x in df.index]
+    return df
+
 def _make_lexi_vars(df):
     df = df.copy()
     #make features lists
-    poss_features = [x for x in FEATURES  if x.endswith('poss') and not x.startswith('verb')]
-    cert_features = [x for x in FEATURES if x.endswith('cert') and not x.startswith('verb')]
+    poss_features = [x for x in FEATURES  if x.endswith('poss')]
+    cert_features = [x for x in FEATURES if x.endswith('cert')]
+    
+    #lexi_sumary
+    lexi_poss_features = [x for x in poss_features if not x.startswith('verb')]
+    lexi_cert_features = [x for x in cert_features if not x.startswith('verb')]
+    
     #apply
-    df['lexi_poss'] = [1 if any(feature == 1 for feature in df.loc[x,poss_features])\
+    df['uncertain'] = [1 if any(feature == 1 for feature in df.loc[x,poss_features])\
                        else 0 for x in df.index]
-    df['lexi_cert'] = [1 if any(feature == 1 for feature in df.loc[x,cert_features])\
+    df['certain'] = [1 if any(feature == 1 for feature in df.loc[x,cert_features])\
+                       else 0 for x in df.index]
+    
+    df['lexi_poss'] = [1 if any(feature == 1 for feature in df.loc[x,lexi_poss_features])\
+                       else 0 for x in df.index]
+    df['lexi_cert'] = [1 if any(feature == 1 for feature in df.loc[x,lexi_cert_features])\
                        else 0 for x in df.index]
     return df
 
 def _make_no_code(df):
     df = df.copy()
     #apply
-    df['no_code'] = ['coded' if any(feature == 1 for feature in df.loc[x,MAIN_FEATURES])\
+    df['no_code'] = ['coded' if any(feature == 1 for feature in df.loc[x,FEATURES])\
                        else 'uncoded' for x in df.index]
     return df
 
 def _tok_return(doc):
     toks =  [token.text.lower() for token in doc if\
             not token.is_punct]
+    toks = [stok for tok in toks for stok in tok.split("'")]
     return toks
 
 def _is_negated(doc):
@@ -107,6 +151,12 @@ def _is_negated(doc):
     else:
         return 0
     
+def _append_last_sentence(df):
+    df = df.copy()
+    df['final_sentence'] = df.spacy_doc #pointless step implementing to test for better accuracy
+    #df['final_sentence'] = df.spacy_doc.apply(lambda doc: [sent for sent in doc.sents][-1])
+    return df
+
 def _check_words(response):
     scores = {}
     for feature,w_list in _word_list.items():
@@ -134,9 +184,15 @@ def _debug_check_words(response,raw_text=False,lang='dutch'):
         else:
             scores[feature+'_debug'] = ''
     return scores
-    
 
-def prepare(df,**kwargs):
+def add_suffix(df,suffix):
+    df = df.copy()
+    for feature in ALL_FEATURES:
+        df[[feature+suffix]] = df[[feature]]
+    df = df.drop(ALL_FEATURES,1)
+    return df
+
+def prepare(df,lang_col,text_col):
     """ append two columns to a pandas dataframe containing at least two columns, one
     definining language in ['english','dutch','german'], the second containing str 
     natural language responses. Appended columns are 'spacy_doc', which is a
@@ -148,13 +204,13 @@ def prepare(df,**kwargs):
     """
     df = df.copy()
     #clean docs with spacy
-    df = _append_spacy_docs(df,**kwargs)
-    df = _append_last_sentence(df)
+    df = _append_spacy_docs(df,lang_col,text_col)
+    df  = _append_last_sentence(df)
     df['response_clean'] = df.final_sentence.apply(lambda x: _tok_return(x))
     return df
 
 
-def score(df,lang_col='language',process_col='final_sentence',debug=False):
+def score(df,lang_col,debug,process_col='final_sentence'):
     """ Append columns for each of the features in ftr.word_lists._FEATURES. Columns are in [0,1], and define whether ftr.word_lists._FEATURES_i is present in df[process_col]
     :param df: a pandas.DataFrame() object
     :param clean_spacy: boolean, default == True. If True, spacy docs in df.final_sentence and df.spacy_doc will be dropped from df, if False, these will be kept.
@@ -180,7 +236,7 @@ def score(df,lang_col='language',process_col='final_sentence',debug=False):
         dy = pd.concat([dy,dy[process_col].apply(lambda doc: pd.Series(_check_words(doc)))],axis=1)
         dy['negated'] = dy[process_col].apply(lambda doc: _is_negated(doc))
         #add lists of hit words to debug if necessary
-        if debug == True:
+        if debug:
             dy = pd.concat([dy,dy[process_col].apply(lambda doc: pd.Series(_debug_check_words(doc)))],axis=1)
         #append together
         dx = dx.append(dy)       
@@ -195,13 +251,16 @@ def apply_dominance(df):
     :return: pd.DataFrame()
     """
     df = df.copy()
-    df = _present_dom(df)
     df = _future_dom(df)
+    df = _aspect_dom(df)
+    df = _past_dom(df)
+    df = _present_dom(df)
+    df = _modal_exclude(df)
     df = _make_lexi_vars(df)
     df = _make_no_code(df)
     return df
     
-def classify_df(df,debug=False,**kwargs):
+def classify_df(df,lang_col='language',text_col='response',suffix=None,debug=False,**kwargs):
     """ Sequentially call prepare(df), score(df), and apply_dominance(df)
     :param df: a pandas.DataFrame() object which MUST match criteria described in prepare() description.
     :param **kwargs: any key_word arguments passable to prepare() or score(), 
@@ -209,11 +268,15 @@ def classify_df(df,debug=False,**kwargs):
     :return: pd.Dataframe()
     """
     df = df.copy()
-    df = prepare(df,**kwargs)
+    df = prepare(df,lang_col,text_col)
     #debug by returning the 'hit' words
-    df = score(df,debug=debug)
+    df = score(df,lang_col,debug,**kwargs)
     #apply dominance
     df = apply_dominance(df)
+    if suffix:
+        df = add_suffix(df,suffix)
+    df = _clean_non_applicable(df,lang_col)
+    df.index = range(len(df.index))
     return df
     
 def clean_spacy(df):
